package de.tu_darmstadt.tk.SmartHomeNetworkSim.control;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Model;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;

/**
 * Controller which allows interaction with the Model and Simulation
 *
 * @author Andreas T. Meyer-Berg
 */
public class Controller {
	/**
	 * {@link Model} which stores the smart home model
	 */
	Model model;

	public Controller(Model model) {
		this.model = model;
	}

	/**
	 * Moves the SmartDevice device to the specified location, if it exists
	 * 
	 * @param device
	 *            the device to move
	 * @param x
	 *            new x position
	 * @param y
	 *            new y position
	 * @param z
	 *            new z postiion
	 */
	public void moveSmartDevice(SmartDevice device, int x, int y, int z) {
		for (SmartDevice d : model.getDevices()) {
			if (d == device) {
				d.setX(x);
				d.setY(y);
				d.setZ(z);
			}
		}
	}

	/**
	 * Sets the Dimension of the model
	 * 
	 * @param width
	 *            the width to set
	 * @param height
	 *            the height to set
	 * @param depth
	 *            the depth to set
	 */
	public void setDimension(int width, int height, int depth) {
		setDimension(width, height, depth, false);
	}

	/**
	 * Sets the Dimension of the model and moves SmartDevices to new relative
	 * positions
	 * 
	 * @param width
	 *            the width to set
	 * @param height
	 *            the height to set
	 * @param depth
	 *            the depth to set
	 * @param true
	 *            if smartDevice positions should be stretched
	 */
	public void setDimension(int width, int height, int depth, boolean stretchModel) {
		//Dont allow to small Models
		if (width < 400)
			width = 400;
		if (height < 400)
			height = 400;
		if (depth < 400)
			depth = 400;

		if (stretchModel) {
			/**
			 * Factor that changes the x position. Example: 1 -> positions stay
			 * the same 0.5 -> x position is halved 2 -> x position is doubled
			 */
			double width_factor = ((double) width) / ((double) model.getWidth());
			/**
			 * Factor that changes the y position. Example: 1 -> positions stay
			 * the same 0.5 -> y position is halved 2 -> y position is doubled
			 */
			double height_factor = ((double) height) / ((double) model.getHeight());
			/**
			 * Factor that changes the z position. Example: 1 -> positions stay
			 * the same 0.5 -> z position is halved 2 -> z position is doubled
			 */
			double depth_factor = ((double) depth) / ((double) model.getDepth());
			// Update all device positions
			for (SmartDevice d : model.getDevices()) {
				if (width_factor != 1.0)
					d.setX(scalePos(d.getX(), width_factor, width));
				if (height_factor != 1.0)
					d.setY(scalePos(d.getY(), height_factor, height));
				if (depth_factor != 1.0)
					d.setZ(scalePos(d.getZ(), depth_factor, depth));
			}
		}
		model.setWidth(width);
		model.setHeight(height);
		model.setDepth(depth);
	}
	
	/**
	 * Calculates the new scaled position, which should radius < newPost < upperBound - radius
	 * @param oldPosition previous Position that should be scaled
	 * @param factor how much it is stretched/shrunk
	 * @param upperBound upper Bound of the frame, which should not be exceeded
	 * @return newPosition that was calculated
	 */
	private int scalePos(int oldPosition, double factor, int upperBound){
		/**
		 * New Position that should be validated and returned
		 */
		int newPosition = (int) Math.round(factor * oldPosition);
		//Update if it moves out of the frame
		if(newPosition<model.getDevice_visualization_radius())
			newPosition = model.getDevice_visualization_radius();
		if(newPosition>=upperBound-model.getDevice_visualization_radius())
			newPosition = upperBound-model.getDevice_visualization_radius();
		
		return newPosition;
	}
	
	/**
	 * Adds SmartDevice to the Model
	 * @param sd
	 */
	public void addSmartDevice(SmartDevice sd){
		model.addDevices(sd);
		//validate Position
		sd.setX(scalePos(sd.getX(), 1.0, model.getWidth()));
		sd.setY(scalePos(sd.getY(), 1.0, model.getHeight()));
		sd.setZ(scalePos(sd.getZ(), 1.0, model.getDepth()));
	}
}
