package de.tu_darmstadt.tk.SmartHomeNetworkSim.view;

import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.event.MouseInputListener;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.control.Controller;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Model;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;

/**
 * Listener which detects User Interaction with the {@link VisualisationPanel},
 * stores interaction information, triggers the controller and helps
 * visualisation of the interaction
 * 
 * @author Andreas T. Meyer-Berg
 */
public class VisualisationInteractor implements MouseInputListener, MouseMotionListener, ActionListener {

	/**
	 * Smart Home model which should be interacted with
	 */
	private Model model;

	/**
	 * Controller to notify when the model changes
	 */
	private Controller control;

	/**
	 * Panel which is observed
	 */
	private VisualisationPanel panel;
	
	/**
	 * Menu which is shown on right clicks
	 */
	private JPopupMenu rightClickMenu;
	
	/**
	 * RightClick MenuItem for SmartDevice creation
	 */
	private JMenuItem itemCreate;
	
	/**
	 * RightClick MenuItem for SmartDevice deletion
	 */
	private JMenuItem itemDelete;
	
	/**
	 * SmartDevice that is dragged on Screen
	 */
	public SmartDevice dragged = null;

	/**
	 * Current x Position of the dragged device
	 */
	public int dragged_x;

	/**
	 * Current y Position of the dragged device
	 */
	public int dragged_y;

	/**
	 * Creates a new VisualisationInteractor
	 * @param model
	 * @param controller
	 */
	public VisualisationInteractor(Model model, Controller controller, VisualisationPanel panel) {
		this.model = model;
		this.control = controller;
		this.panel = panel;
		this.rightClickMenu = new JPopupMenu();
		itemCreate = new JMenuItem("Create SmartDevice");
		itemDelete = new JMenuItem("Delete");
		rightClickMenu.add(itemCreate);
		rightClickMenu.add(itemDelete);
		this.panel.add(rightClickMenu);
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		/**
		 * SmartDevice that was clicked
		 */
		SmartDevice clickedOn = null;
		// Find the clicked SmartDevice
		for (SmartDevice d : model.getDevices()) {
			if (Math.abs(d.getX() - e.getX()) < panel.getVisualisationRadius()
					&& Math.abs(d.getY() - e.getY()) < panel.getVisualisationRadius()) {
				clickedOn = d;
			}
		}
		//If Rightclick
		if(e.getButton()==MouseEvent.BUTTON3) {
			//Show the RightClickMenu
			showRightClickMenu(clickedOn);	
		}
			
	}

	/**
	 * Shows the RightClick Menu on the Visualisation Panel, with Options for the given SmartDevice clickedOn, 
	 * if clickedOn is null, the RightClick Menu contains Options for creation of new SmartDevices.
	 * The Menu will be shown at the Mouse position on the VisualisationPanel. 
	 * @param clickedOn Device which was clicked, null if no
	 */
	protected void showRightClickMenu(SmartDevice clickedOn) {
		/**
		 * Mouse Position on the VisualisationPanel
		 */
		Point mousePos = panel.getMousePosition();
		//Just execute if Mouse Position is on the Panel
		if(mousePos != null){
			if(clickedOn == null){		
				itemCreate.setEnabled(true);
				itemDelete.setEnabled(false);
			}
			else{
				itemCreate.setEnabled(false);
				itemDelete.setEnabled(true);
			}
			rightClickMenu.show(panel, mousePos.x, mousePos.y);
			rightClickMenu.setEnabled(true);
			rightClickMenu.setVisible(true);
		}
	}

	@Override
	public void mousePressed(MouseEvent e) {
		// Check if SmartDevice was clicked
		for (SmartDevice d : model.getDevices()) {
			if (Math.abs(d.getX() - e.getX()) < panel.getVisualisationRadius()
					&& Math.abs(d.getY() - e.getY()) < panel.getVisualisationRadius()) {
				// Recognize possible drag operation
				if (e.getButton() == MouseEvent.BUTTON1) {
					dragged = d;
					dragged_x = d.getX();
					dragged_y = d.getY();
				}
			}
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// Finish drag operation
		if (dragged != null && e.getButton() == MouseEvent.BUTTON1
				&& (dragged.getX() != dragged_x || dragged.getY() != dragged_y)) {
			control.moveSmartDevice(dragged, dragged_x, dragged_y, dragged.getZ());
			panel.repaint();
		}
		dragged = null;
	}

	@Override
	public void mouseEntered(MouseEvent e) {

	}

	@Override
	public void mouseExited(MouseEvent e) {

	}

	@Override
	public void actionPerformed(ActionEvent e) {

	}

	@Override
	public void mouseDragged(MouseEvent e) {
		// move dragged object on screen along y Axis
		if (e.getX() <= panel.getVisualisationRadius())
			dragged_x = panel.getVisualisationRadius();
		else if (e.getX() >= model.getWidth() - panel.getVisualisationRadius())
			dragged_x = model.getWidth() - panel.getVisualisationRadius();
		else
			dragged_x = e.getX();
		// move dragged object on screen along y Axis
		if (e.getY() <= panel.getVisualisationRadius())
			dragged_y = panel.getVisualisationRadius();
		else if (e.getY() >= model.getHeight() - panel.getVisualisationRadius())
			dragged_y = model.getHeight() - panel.getVisualisationRadius();
		else
			dragged_y = e.getY();
		// repaint the drag
		if (dragged != null)
			panel.repaint();
	}

	@Override
	public void mouseMoved(MouseEvent e) {
	}

}
